"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    Static: function() {
        return Static;
    },
    Relative: function() {
        return Relative;
    },
    Absolute: function() {
        return Absolute;
    },
    Fixed: function() {
        return Fixed;
    },
    Sticky: function() {
        return Sticky;
    }
});
var _extends = require("@swc/helpers/lib/_extends.js").default;
var _interopRequireDefault = require("@swc/helpers/lib/_interop_require_default.js").default;
var _objectWithoutProperties = require("@swc/helpers/lib/_object_without_properties.js").default;
var _clsx = /*#__PURE__*/ _interopRequireDefault(require("clsx"));
var _react = /*#__PURE__*/ _interopRequireDefault(require("react"));
var _utils = require("../utils");
var _box = require("../Box");
var Direction;
(function(Direction) {
    Direction["INSET"] = "inset";
    Direction["Y"] = "inset-y";
    Direction["X"] = "inset-x";
    Direction["LEFT"] = "left";
    Direction["RIGHT"] = "right";
    Direction["TOP"] = "top";
    Direction["BOTTOM"] = "bottom";
})(Direction || (Direction = {}));
function PositionBase(props) {
    var className = props.className, position = props.position, zIndex = props.zIndex, inset = props.inset, x = props.x, y = props.y, top = props.top, bottom = props.bottom, left = props.left, right = props.right, boxProps = _objectWithoutProperties(props, [
        "className",
        "position",
        "zIndex",
        "inset",
        "x",
        "y",
        "top",
        "bottom",
        "left",
        "right"
    ]);
    // TODO: Support responsive
    var positionDirectionClassNames = resolveClassName({
        inset: inset,
        x: x,
        y: y,
        top: top,
        bottom: bottom,
        left: left,
        right: right
    });
    return /*#__PURE__*/ _react.default.createElement(_box.Box, _extends({
        className: (0, _clsx.default)((0, _utils.responsive)(_utils.resolveValue, position), positionDirectionClassNames, (0, _utils.responsive)(_utils.resolveValue, zIndex), className)
    }, boxProps));
}
function Static(props) {
    return /*#__PURE__*/ _react.default.createElement(PositionBase, _extends({
        position: "static"
    }, props));
}
function Relative(props) {
    return /*#__PURE__*/ _react.default.createElement(PositionBase, _extends({
        position: "relative"
    }, props));
}
function Absolute(props) {
    return /*#__PURE__*/ _react.default.createElement(PositionBase, _extends({
        position: "absolute"
    }, props));
}
function Fixed(props) {
    return /*#__PURE__*/ _react.default.createElement(PositionBase, _extends({
        position: "fixed"
    }, props));
}
function Sticky(props) {
    return /*#__PURE__*/ _react.default.createElement(PositionBase, _extends({
        position: "sticky"
    }, props));
}
function getClassName(dir, value) {
    if (value == null) return "";
    var isNegative = typeof value === "number" && value < 0 || typeof value === "string" && value[0] === "-";
    var normalizedValue = typeof value === "number" ? Math.abs(value) : value.substring(1);
    var className = "".concat(dir, "-").concat(normalizedValue);
    return isNegative ? "-".concat(className) : className;
}
function resolveClassName(direction) {
    if ("isnet" in direction) return getClassName(Direction.INSET, direction);
    // Always resolve to the most specific
    var top = "y" in direction ? direction.y : direction.top;
    var bottom = "y" in direction ? direction.y : direction.bottom;
    var left = "x" in direction ? direction.x : direction.left;
    var right = "x" in direction ? direction.x : direction.right;
    if (top === bottom && left === right && top === left) return getClassName(Direction.INSET, top);
    var classNames = [];
    if (top === bottom) classNames.push(getClassName(Direction.Y, top));
    else classNames.push(getClassName(Direction.TOP, top), getClassName(Direction.BOTTOM, bottom));
    if (left === right) classNames.push(getClassName(Direction.X, left));
    else classNames.push(getClassName(Direction.LEFT, left), getClassName(Direction.RIGHT, right));
    return (0, _clsx.default)(classNames);
}

//# sourceMappingURL=Position.js.map